@@ -2,7 +2,7 @@ package Plack::Middleware::Profiler::NYTProf;
 use strict;
 use warnings;
 use parent qw(Plack::Middleware);
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 use Plack::Util::Accessor qw(
     enable_profile
@@ -25,6 +25,8 @@ use File::Which;
 use constant PROFILE_ID       => 'psgix.profiler.nytprof.reqid';
 use constant PROFILER_ENABLED => 'psgix.profiler.nytprof.enabled';
 
+my %PROFILER_SETUPED;
+
 sub prepare_app {
     my $self = shift;
 
@@ -93,12 +95,10 @@ sub _setup_profiling_hooks {
 
 }
 
-my %PROFILER_SETUPED;
-
 sub call {
     my ( $self, $env ) = @_;
 
-    $self->_setup_profiler unless $PROFILER_SETUPED{$$};
+    $self->_setup_profiler($env);
     $self->start_profiling_if_needed($env);
 
     my $res = $self->app->($env);
@@ -145,7 +145,14 @@ sub stop_profiling_and_report_if_needed {
 }
 
 sub _setup_profiler {
-    my $self = shift;
+    my ( $self, $env ) = @_;
+
+    my $pid = $$;
+    return if $PROFILER_SETUPED{$pid};
+    $PROFILER_SETUPED{$pid} = 1;
+
+    my $is_profiler_enabled = $self->enable_profile->($env);
+    return unless $is_profiler_enabled;
 
     $ENV{NYTPROF} = $self->env_nytprof || 'start=no';
 
@@ -153,7 +160,6 @@ sub _setup_profiler {
     # so, we load Devel::NYTProf here.
     require Devel::NYTProf;
     DB::disable_profile();
-    $PROFILER_SETUPED{$$} = 1;
 }
 
 sub start_profiling {
